#include <Arduino.h>
#include <ESP8266WiFi.h>
#include <PubSubClient.h>

#define REDPIN1 5
#define GREENPIN1 4
#define BLUEPIN1 13

#define REDPIN2 12
#define GREENPIN2 14
#define BLUEPIN2 16

#define MQTT_TOPIC_START "led1"
#define MQTT_USER ""
#define MQTT_PASS ""
#define MQTT_BROKER "mqtt.example.com"
#define MQTT_CLIENT_ID "LED-Schrank-1"

const char* SSID = "WIFISSID";
const char* PSK = "WIFIPASS";

WiFiClient espClient;
PubSubClient client(espClient);



struct LEDStripe {
  String MOD = "off"; //Which mode is used "off", "fade", "on"
  String OLD_MOD = "off";
  int SPEED = 3000;
  int RED = 0;
  int GREEN = 0;
  int BLUE = 0;
  int LOOP = 0;
  int LOOPCOUNT = 0;
  int MOD_STEP = 0;
};

int Sync = 0; //Sync LED2 on the value of LED1

LEDStripe led1;
LEDStripe led2;

int debugLoop = 0;

//Blink
int fadeLevel = 0;
int fadeMode = 0;
void fade() {
  if(fadeMode == 0) {
    fadeLevel = fadeLevel + 10;
    if(fadeLevel > 1023) {
      fadeLevel = 1024;
      fadeMode = 1;
    }
  }
  if(fadeMode == 1) {
    fadeLevel = fadeLevel - 10;
    if(fadeLevel <= 0) {
      fadeMode = 0;
      fadeLevel = 0;
    }
  }
  analogWrite(LED_BUILTIN, fadeLevel);
}
void blink_short() {
  digitalWrite(LED_BUILTIN, LOW);
  delay(200);
  digitalWrite(LED_BUILTIN, HIGH);
  delay(200);
}
void blink_long() {
  digitalWrite(LED_BUILTIN, LOW);
  delay(1000);
  digitalWrite(LED_BUILTIN, HIGH);
  delay(200);
}
void blink(int status) {
  if(status == 1) {
    blink_short();
    blink_short();
    blink_short();
    blink_short();
  }
  if(status == 2) {
    blink_short();
    blink_short();
    blink_long();
    blink_long();
  }
  if(status == 3) {
    blink_long();
    blink_long();
    blink_long();
    blink_long();
  }
}

void connectWiFi() {
  digitalWrite(LED_BUILTIN, LOW);
  Serial.println("Start WiFi Connection");
  int wifiConnectionCount = 0;
  WiFi.begin(SSID, PSK);
  while (WiFi.status() != WL_CONNECTED) {
    delay(5);
    fade();
    Serial.print(".");
    wifiConnectionCount++;
    if(wifiConnectionCount > 1000) {
      Serial.println("No Connection found");
      wifiConnectionCount = 0;
      blink(2);
    }
  }
  digitalWrite(LED_BUILTIN, HIGH);
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
}





String getTopicNameString(String topic) {
  String topicName = String(MQTT_TOPIC_START)+String(topic);
  Serial.println("TopicName: "+topicName);
  return topicName;
}

char* getTopicName(String topic) {
  String topicName = getTopicNameString(topic);
  Serial.println("TopicName: "+topicName);
  int str_len = topicName.length() + 1; 
  char cc[str_len];
  topicName.toCharArray(cc, str_len);
  Serial.println(cc);
  return cc;
}

void connectMQTT() {
  digitalWrite(LED_BUILTIN, LOW);
  Serial.print("Reconnecting...");
  if (!client.connect(MQTT_CLIENT_ID, MQTT_USER, MQTT_PASS, getTopicName("/status"), 1, true, "0")) {
    Serial.print("failed, rc=");
    Serial.print(client.state());
    Serial.println(" retrying in 5 seconds");
    digitalWrite(LED_BUILTIN, HIGH);
    delay(2000);
    blink(1);
    delay(3000);
  } 
  digitalWrite(LED_BUILTIN, HIGH);
  Serial.println("Subscribe");
  //client.subscribe(getTopicName("/#"));
  client.subscribe("led2/#");
  client.publish(getTopicName("/status"), "1", true);
}



//Actions
void pulse_white_three_times() {
  LEDStripe tmpLed();
}

//Modes
//LED Fade
void fade(LEDStripe *led) {
  if(led->MOD_STEP == 0) {
    led->RED ++;
    led->GREEN = 0;
    led->BLUE = 255;
    if(led->RED >= 255) {
      led->MOD_STEP++;
    }
  }
  if(led->MOD_STEP == 1) {
    led->RED = 255;
    led->GREEN = 0;
    led->BLUE --;
    if(led->BLUE <= 0) {
      led->MOD_STEP++;
    }
  }
  if(led->MOD_STEP == 2) {
    led->RED = 255;
    led->GREEN ++;
    led->BLUE = 0;
    if(led->GREEN >= 255) {
      led->MOD_STEP++;
    }
  }
  if(led->MOD_STEP == 3) {
    led->RED --;
    led->GREEN = 255;
    led->BLUE = 0;
    if(led->RED <= 0) {
      led->MOD_STEP++;
    }
  }
  if(led->MOD_STEP == 4) {
    led->RED = 0;
    led->GREEN = 255;
    led->BLUE ++;
    if(led->BLUE >= 255) {
      led->MOD_STEP++;
    }
  }
  if(led->MOD_STEP == 5) {
    led->RED = 0;
    led->GREEN --;
    led->BLUE = 255;
    if(led->GREEN <= 0) {
      led->MOD_STEP = 0;
    }
  }
}

void writeToLedStrips(LEDStripe *l1, LEDStripe *l2) {
  analogWrite(REDPIN1, l1->RED);
  analogWrite(REDPIN2, l2->RED);
  analogWrite(GREENPIN1, l1->GREEN);
  analogWrite(GREENPIN2, l2->GREEN);
  analogWrite(BLUEPIN1, l1->BLUE);
  analogWrite(BLUEPIN2, l2->BLUE);
  debugLoop++;
  if(debugLoop > 500) {
    Serial.println("LED1: "+String(l1->RED)+"|"+String(l1->GREEN)+"|"+String(l1->BLUE));
    Serial.println("LED2: "+String(l2->RED)+"|"+String(l2->GREEN)+"|"+String(l2->BLUE));
    debugLoop = 0;
  }
  
}

void callback(char* topic, byte* payload, unsigned int length) {
  Serial.println("CALLBACK");
  String message = "";
  for (int i=0;i<int(length);i++) {
    message = message + String((char)payload[i]);
  }
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("] ");
  Serial.print(message);
  Serial.println();
  Serial.println(getTopicNameString("/1/mode"));
  if(String(topic) == getTopicNameString("/1/mode")) {
    led1.MOD = message;
    Serial.println("Set Mode");
  }
  if(String(topic) == getTopicNameString("/2/mode")) {
    led2.MOD = message;
  }
  if(String(topic) == getTopicNameString("/1/speed")) {
    led1.SPEED = message.toInt();
  }
  if(String(topic) == getTopicNameString("/2/speed")) {
    led2.SPEED = message.toInt();
  }

  if(String(topic) == getTopicNameString("/1/color")) {
    led1.RED = message.substring(0,3).toInt();
    led1.GREEN = message.substring(3,6).toInt();
    led1.BLUE = message.substring(6,9).toInt();
  }
  if(String(topic) == getTopicNameString("/2/color")) {
    led2.RED = message.substring(0,3).toInt();
    led2.GREEN = message.substring(3,6).toInt();
    led2.BLUE = message.substring(6,9).toInt();
  }
  if(String(topic) ==  getTopicNameString("/sync")) {
    Sync = message.toInt();
  }
  Serial.println("Message done");
}

void prepareMQTT() {
  digitalWrite(LED_BUILTIN, LOW);
  Serial.println("Prepare MQTT Client");

  client.setServer(MQTT_BROKER, 1883);
  client.setCallback(callback);
  digitalWrite(LED_BUILTIN, HIGH);
  delay(200);
}

void setup() {
  Serial.begin(115200);
  pinMode(LED_BUILTIN, OUTPUT);
  Serial.println();
  Serial.println("Start LED-Controller");
  connectWiFi();
  prepareMQTT();
  connectMQTT();  

  //init LED Strips
  pinMode(REDPIN1, OUTPUT);
  pinMode(REDPIN2, OUTPUT);
  pinMode(GREENPIN1, OUTPUT);
  pinMode(GREENPIN2, OUTPUT);
  pinMode(BLUEPIN1, OUTPUT);
  pinMode(BLUEPIN2, OUTPUT);

  //turn led Strips off
  analogWrite(REDPIN1, 0);
  analogWrite(REDPIN2, 0);
  analogWrite(GREENPIN1, 0);
  analogWrite(GREENPIN2, 0);
  analogWrite(BLUEPIN1, 0);
  analogWrite(BLUEPIN2, 0);
}


void loop() {
  if (!client.connected()) {
    connectMQTT();
  }
  client.loop();

  if(led1.MOD == "fade") {
    fade(&led1);
  }

  if(Sync == 1) {
    led2.RED = led1.RED;
    led2.GREEN = led1.GREEN;
    led2.BLUE = led1.BLUE;
  } else {
    if(led2.MOD == "fade") {
      fade(&led2);
    }
  }

  //Send calculated colors to led-strips
  writeToLedStrips(&led1, &led2);
  delay(10);
}

